@@ -5,10 +5,12 @@ import pytest
 import requests
 import zalando_aws_cli.api
 from click.testing import CliRunner
+from urllib.parse import urlencode
 
 import zalando_kubectl.main
 import zalando_kubectl.registry
 import zalando_kubectl.utils
+import zalando_kubectl.access_request
 from zalando_kubectl.main import click_cli
 
 
@@ -155,6 +157,8 @@ def mock_http_post(monkeypatch, expected_url, expected_json, response):
 
 def mock_http_get(monkeypatch, expected_url, expected_json, response):
     def mock_fn(url, json=None, **_kwargs):
+        if "params" in _kwargs:
+            url = url + "?" + urlencode(_kwargs["params"])
         assert url == expected_url
         if expected_json:
             assert json == expected_json
@@ -224,6 +228,214 @@ def test_request_manual_access_no_message(monkeypatch):
     assert_cli_failed("cluster-access", "request")
 
 
+def mock_get_auth_token(monkeypatch):
+    monkeypatch.setattr(
+        "zalando_kubectl.kube_config.get_auth_token", lambda env, cluster, force_refresh=False: "YXV0aHRva2VuCg=="
+    )
+
+
+def mock_get_current_context(monkeypatch, context_name):
+    monkeypatch.setattr("zalando_kubectl.kube_config.get_current_context", lambda: context_name)
+
+
+def test_request_manual_access_with_okta_approved(monkeypatch):
+    mock_config(monkeypatch)
+    mock_get_cluster(monkeypatch, {"environment": "production"})
+    mock_get_auth_token(monkeypatch)
+    mock_get_current_context(monkeypatch, "production_cluster")
+    zalando_kubectl.access_request._remove_state_file("privileged", "production_cluster")
+
+    expected_json = {
+        "access_role": "default",
+        "account_name": "production_cluster",
+        "business_justification": "foo bar",
+    }
+    response = {
+        "result": {
+            "request_key": "85625",
+            "_links": {
+                "request_status": {
+                    "href": "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?request_key=85625"
+                },
+                "approval_ui": {"href": "https://zalando-dev.saviyntcloud.com/ECMv6/review/requestApproval/6030927"},
+            },
+        }
+    }
+    mock_http_post(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/privileged/cloud-infrastructure/aws/account",
+        expected_json,
+        response,
+    )
+
+    response2 = [
+        {
+            "request_status": "APPROVED",
+        }
+    ]
+    mock_http_get(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?request_key=85625",
+        None,
+        response2,
+    )
+
+    assert_cli_successful("cluster-access", "request", "--okta", "foo", "bar")
+    assert not zalando_kubectl.access_request._has_state_file("privileged", "production_cluster")
+
+
+def test_request_manual_access_with_okta_pending(monkeypatch):
+    mock_config(monkeypatch)
+    mock_get_cluster(monkeypatch, {"environment": "production"})
+    mock_get_auth_token(monkeypatch)
+    mock_get_current_context(monkeypatch, "production_cluster")
+    zalando_kubectl.access_request._remove_state_file("privileged", "production_cluster")
+
+    expected_json = {
+        "access_role": "default",
+        "account_name": "production_cluster",
+        "business_justification": "foo bar",
+    }
+    response = {
+        "result": {
+            "request_key": "85625",
+            "_links": {
+                "request_status": {
+                    "href": "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?request_key=85625"
+                },
+                "approval_ui": {"href": "https://zalando-dev.saviyntcloud.com/ECMv6/review/requestApproval/6030927"},
+            },
+        }
+    }
+    mock_http_post(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/privileged/cloud-infrastructure/aws/account",
+        expected_json,
+        response,
+    )
+
+    response2 = [
+        {
+            "request_status": "PENDING",
+        }
+    ]
+    mock_http_get(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?request_key=85625",
+        None,
+        response2,
+    )
+
+    assert_cli_successful("cluster-access", "request", "--okta", "--timeout", "15", "foo", "bar")
+    assert zalando_kubectl.access_request._has_state_file("privileged", "production_cluster")
+
+
+def test_list_access_requests_with_okta(monkeypatch):
+    mock_config(monkeypatch)
+    mock_get_cluster(monkeypatch, {"environment": "production"})
+    mock_get_auth_token(monkeypatch)
+    mock_get_current_context(monkeypatch, "production_cluster")
+
+    response = [
+        {
+            "request_status": "PENDING",
+        }
+    ]
+    mock_http_get(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?account_name=production_cluster",
+        None,
+        response,
+    )
+    assert_cli_successful("cluster-access", "list", "--okta")
+
+    mock_http_get(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?account_name=production_cluster&request_key=85625",
+        None,
+        response,
+    )
+    assert_cli_successful("cluster-access", "list", "--okta", "--request-key", "85625")
+
+
+def test_approve_manual_access_with_okta(monkeypatch):
+    mock_config(monkeypatch)
+    mock_get_cluster(monkeypatch, {"environment": "production"})
+    mock_get_auth_token(monkeypatch)
+
+    response = [
+        {
+            "requestor": "team-velma+privileged-access-pen-test-requestor@zalando.de",
+            "account_name": "production_cluster",
+            "business_justification": "foo bar",
+            "request_status": "PENDING",
+        }
+    ]
+    mock_http_get(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?request_key=85625",
+        None,
+        response,
+    )
+
+    expected_json = {
+        "account_name": "production_cluster",
+        "business_justification": "foo bar",
+        "access_role": "default",
+        "approver_comment": "",
+        "decision": "APPROVED",
+        "request_key": "85625",
+        "requestor": "team-velma+privileged-access-pen-test-requestor@zalando.de",
+    }
+    mock_http_post(
+        monkeypatch,
+        "http://access.zalan.do/v1/approve/tech/privileged/cloud-infrastructure/aws/account",
+        expected_json,
+        None,
+    )
+
+    assert_cli_successful("cluster-access", "approve", "--okta", "--yes", "85625")
+
+
+def test_approve_manual_access_with_okta_already_approved(monkeypatch):
+    mock_config(monkeypatch)
+    mock_get_cluster(monkeypatch, {"environment": "production"})
+    mock_get_auth_token(monkeypatch)
+
+    response = [
+        {
+            "requestor": "team-velma+privileged-access-pen-test-requestor@zalando.de",
+            "account_name": "production_cluster",
+            "business_justification": "foo bar",
+            "request_status": "APPROVED",
+        }
+    ]
+    mock_http_get(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?request_key=85625",
+        None,
+        response,
+    )
+
+    expected_json = {
+        "account_name": "production_cluster",
+        "business_justification": "foo bar",
+        "access_role": "default",
+        "approver_comment": "",
+        "decision": "APPROVED",
+        "request_key": "85625",
+        "requestor": "team-velma+privileged-access-pen-test-requestor@zalando.de",
+    }
+    mock_http_post(
+        monkeypatch,
+        "http://access.zalan.do/v1/approve/tech/privileged/cloud-infrastructure/aws/account",
+        expected_json,
+        requests.exceptions.HTTPError(412),
+    )
+
+    assert_cli_successful("cluster-access", "approve", "--okta", "--yes", "85625")
+
+
 @pytest.mark.parametrize("get_cluster_success", [False, True])
 def test_request_emergency_access(monkeypatch, get_cluster_success):
     mock_config(monkeypatch)
@@ -301,6 +513,15 @@ def test_request_emergency_access_test_cluster(monkeypatch):
     assert_cli_failed("cluster-access", "request", "--emergency", "-i", "1111", "foo", "bar")
 
 
+def test_request_manual_access_with_okta_test_cluster(monkeypatch):
+    mock_config(monkeypatch)
+    mock_get_cluster(monkeypatch, {"environment": "test"})
+    mock_get_auth_token(monkeypatch)
+    mock_get_current_context(monkeypatch, "test_cluster")
+
+    assert_cli_failed("cluster-access", "request", "--okta", "foo", "bar")
+
+
 def test_request_emergency_access_different_user(monkeypatch):
     mock_config(monkeypatch)
     mock_get_api_server_url(monkeypatch)
@@ -355,6 +576,121 @@ def test_request_emergency_access_http_error(monkeypatch):
     assert_cli_failed("cluster-access", "request", "--emergency", "-u" "foo", "-i", "1111", "foo", "bar")
 
 
+def test_request_emergency_access_with_okta_approved(monkeypatch):
+    mock_config(monkeypatch)
+    mock_get_cluster(monkeypatch, {"environment": "production"})
+    mock_get_auth_token(monkeypatch)
+    mock_get_current_context(monkeypatch, "production_cluster")
+    reference_url = "https://zalando.app.opsgenie.com/alert/detail/a6f295c8-9518-4427-a4ff-000000000-000000000/details"
+    zalando_kubectl.access_request._remove_state_file("emergency", "production_cluster")
+
+    expected_json = {
+        "access_role": "default",
+        "account_name": "production_cluster",
+        "business_justification": "foo bar",
+        "reference_url": reference_url,
+    }
+    response = {
+        "result": {
+            "request_key": "85625",
+            "_links": {
+                "request_status": {
+                    "href": "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?request_key=85625"
+                },
+            },
+        }
+    }
+    mock_http_post(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/emergency/cloud-infrastructure/aws/account",
+        expected_json,
+        response,
+    )
+
+    response2 = [
+        {
+            "request_status": "APPROVED",
+        }
+    ]
+    mock_http_get(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?request_key=85625",
+        None,
+        response2,
+    )
+
+    assert_cli_successful("cluster-access", "request", "--okta", "--emergency", "-i", reference_url, "foo", "bar")
+    assert not zalando_kubectl.access_request._has_state_file("emergency", "production_cluster")
+
+
+def test_request_emergency_access_with_okta_pending(monkeypatch):
+    mock_config(monkeypatch)
+    mock_get_cluster(monkeypatch, {"environment": "production"})
+    mock_get_auth_token(monkeypatch)
+    mock_get_current_context(monkeypatch, "production_cluster")
+    reference_url = "https://zalando.app.opsgenie.com/alert/detail/a6f295c8-9518-4427-a4ff-000000000-000000000/details"
+    zalando_kubectl.access_request._remove_state_file("emergency", "production_cluster")
+
+    expected_json = {
+        "access_role": "default",
+        "account_name": "production_cluster",
+        "business_justification": "foo bar",
+        "reference_url": reference_url,
+    }
+    response = {
+        "result": {
+            "request_key": "85625",
+            "_links": {
+                "request_status": {
+                    "href": "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?request_key=85625"
+                },
+            },
+        }
+    }
+    mock_http_post(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/emergency/cloud-infrastructure/aws/account",
+        expected_json,
+        response,
+    )
+
+    response2 = [
+        {
+            "request_status": "PENDING",
+        }
+    ]
+    mock_http_get(
+        monkeypatch,
+        "http://access.zalan.do/v1/requests/tech/cloud-infrastructure/aws/account?request_key=85625",
+        None,
+        response2,
+    )
+
+    assert_cli_successful(
+        "cluster-access", "request", "--okta", "--emergency", "-i", reference_url, "--timeout", "15", "foo", "bar"
+    )
+    assert zalando_kubectl.access_request._has_state_file("emergency", "production_cluster")
+
+
+def test_request_emergency_access_with_okta_invalid_incident(monkeypatch):
+    mock_config(monkeypatch)
+    mock_get_cluster(monkeypatch, {"environment": "production"})
+    mock_get_auth_token(monkeypatch)
+    mock_get_current_context(monkeypatch, "production_cluster")
+
+    assert_cli_failed("cluster-access", "request", "--okta", "--emergency", "-i", "WRONG_LINK", "foo", "bar")
+
+
+def test_request_emergency_access_with_okta_no_message(monkeypatch):
+    mock_config(monkeypatch)
+    mock_get_cluster(monkeypatch, {"environment": "production"})
+    mock_get_auth_token(monkeypatch)
+    mock_get_current_context(monkeypatch, "production_cluster")
+    reference_url = "https://zalando.app.opsgenie.com/alert/detail/a6f295c8-9518-4427-a4ff-000000000-000000000/details"
+
+    assert_cli_failed("cluster-access", "request", "--okta", "--emergency", "-i", reference_url)
+
+
 def mock_access_requests(monkeypatch, *access_requests):
     mock_config(monkeypatch)
     mock_get_api_server_url(monkeypatch)
@@ -545,7 +881,11 @@ def mock_config(monkeypatch):
     monkeypatch.setattr("zign.api.get_token", lambda _x, _y: "mytok")
     monkeypatch.setattr(
         "stups_cli.config.load_config",
-        lambda x: {"cluster_registry": "http://registry.zalan.do"},
+        lambda x: {
+            "cluster_registry": "http://registry.zalan.do",
+            "okta_auth": "http://okta.zalan.do",
+            "privileged_access_api": "http://access.zalan.do",
+        },
     )
 
 
@@ -706,7 +1046,8 @@ def test_cluster_update_block_overwrite(monkeypatch):
 @pytest.mark.parametrize(
     "key_id", [("alias/mycluster-deployment-secret", []), ("custom-key", ["--kms-keyid", "custom-key"])]
 )
-def test_encrypt(monkeypatch, cluster, key_id, strip):
+@pytest.mark.parametrize("okta", [(True, []), (False, ["--okta"]), (True, ["--no-okta"])])
+def test_encrypt(monkeypatch, cluster, key_id, strip, okta):
     mock_config(monkeypatch)
     mock_get_api_server_url(monkeypatch)
 
@@ -721,6 +1062,7 @@ def test_encrypt(monkeypatch, cluster, key_id, strip):
     expected_cluster, cluster_args = cluster
     expected_key, key_id_args = key_id
     plaintext, strip_args = strip
+    use_kms, okta_args = okta
 
     if expected_cluster:
         mock_get_cluster(monkeypatch, test_cluster, expected_id=expected_cluster)
@@ -737,22 +1079,31 @@ def test_encrypt(monkeypatch, cluster, key_id, strip):
     monkeypatch.setattr("zalando_aws_cli.api.get_roles", lambda token: mock_roles)
     monkeypatch.setattr("zalando_kubectl.secrets.create_boto_session", lambda token, account_id, role_name: mock_boto)
 
+    mock_run = MagicMock()
+    mock_run.return_value.stdout = b"deployment-secret:2:test:dGVzdA=="
+    mock_run.return_value.wait.return_value = 0
+    monkeypatch.setattr("subprocess.run", mock_run)
+
     cmdline = ["encrypt"]
     cmdline.extend(key_id_args)
     cmdline.extend(strip_args)
     cmdline.extend(cluster_args)
+    cmdline.extend(okta_args)
 
     result = CliRunner().invoke(click_cli, cmdline, input="foo\n")
 
     expect_success(result, "deployment-secret:2:test:dGVzdA==")
 
     mock_kms = unittest.mock.call.client("kms", test_region)
-    expected_calls = mock_kms.encrypt(KeyId=expected_key, Plaintext=plaintext.encode()).call_list()
+    expected_calls = []
+    if use_kms:
+        expected_calls = mock_kms.encrypt(KeyId=expected_key, Plaintext=plaintext.encode()).call_list()
     assert mock_boto.mock_calls == expected_calls
 
 
 @pytest.mark.parametrize("secret_args", [["deployment-secret:test:dGVzdA=="], ["deployment-secret:2:test:dGVzdA=="]])
-def test_decrypt(monkeypatch, secret_args):
+@pytest.mark.parametrize("okta", [(True, []), (False, ["--okta"]), (True, ["--no-okta"])])
+def test_decrypt(monkeypatch, secret_args, okta):
     mock_config(monkeypatch)
     mock_get_api_server_url(monkeypatch)
 
@@ -764,6 +1115,8 @@ def test_decrypt(monkeypatch, secret_args):
         "region": test_region,
     }
 
+    use_kms, okta_args = okta
+
     def mock_get_cluster_with_params(_config, **params):
         assert params == {"alias": "test"}
         return test_cluster
@@ -778,13 +1131,21 @@ def test_decrypt(monkeypatch, secret_args):
     monkeypatch.setattr("zalando_aws_cli.api.get_roles", lambda token: mock_roles)
     monkeypatch.setattr("zalando_kubectl.secrets.create_boto_session", lambda token, account_id, role_name: mock_boto)
 
+    mock_run = MagicMock()
+    mock_run.return_value.stdout = b"foo\r\n"
+    mock_run.return_value.wait.return_value = 0
+    monkeypatch.setattr("subprocess.run", mock_run)
+
     cmdline = ["decrypt"]
     cmdline.extend(secret_args)
+    cmdline.extend(okta_args)
 
     result = CliRunner().invoke(click_cli, cmdline)
 
     expect_success(result, "foo")
 
     mock_kms = unittest.mock.call.client("kms", test_region)
-    expected_calls = mock_kms.decrypt(CiphertextBlob=b"test").call_list()
+    expected_calls = []
+    if use_kms:
+        expected_calls = mock_kms.decrypt(CiphertextBlob=b"test").call_list()
     assert mock_boto.mock_calls == expected_calls
