@@ -6,6 +6,7 @@ import yaml
 
 from subprocess import PIPE
 from zalando_kubectl.utils import Environment, auth_token
+import zalando_kubectl.access_request
 
 
 KUBECONFIG = os.path.expanduser("~/.kube/config")
@@ -51,27 +52,30 @@ def update(url, alias, token, use_okta):
     return updated_config
 
 
-def get_auth(env: Environment, cluster, stdout=None, stderr=None):
+def get_auth(env: Environment, cluster, force_refresh=False, stdout=None, stderr=None):
     oidc_issuer_url = get_okta_auth_url(env.config)
-    cmd = (
+    cmd = [
         "get-token",
         f"--oidc-issuer-url={oidc_issuer_url}",
         f"--oidc-client-id=kubernetes.cluster.{cluster}",
         "--oidc-extra-scope=email",
         "--oidc-extra-scope=offline_access",
-    )
-    return env.kubelogin.run(cmd, stdout=stdout, stderr=stderr)
+    ]
+    if force_refresh:
+        cmd.append("--force-refresh")
+
+    return env.kubelogin.run(cmd, stdout=stdout, stderr=stderr, forward_context=False, forward_namespace=False)
 
 
 def get_okta_auth_url(config):
     try:
         return config["okta_auth"].rstrip("/")
     except KeyError:
-        raise Exception("Okta auth URL missing, please reconfigure zkubectl")
+        raise Exception("Okta auth URL missing, please reconfigure zkubectl by running `zalando-cli-bundle configure`")
 
 
-def get_auth_token(env, alias):
-    token_bytes = get_auth(env, alias, stdout=PIPE, stderr=PIPE).stdout
+def get_auth_token(env, alias, force_refresh=False):
+    token_bytes = get_auth(env, alias, force_refresh, stdout=PIPE, stderr=PIPE).stdout
     return json.loads(token_bytes.decode("utf-8"))["status"]["token"]
 
 
@@ -79,13 +83,17 @@ def is_okta_user():
     config = read_config()
     try:
         for c in config["contexts"]:
-            return c["context"]["cluster"] == config["current-context"] and "okta" in c["context"]["user"]
+            if c["name"] == config["current-context"]:
+                return "okta" in c["context"]["user"]
+        return False
     except KeyError:
         return False
 
 
-def update_token():
+def update_token(env):
     if is_okta_user():
+        current_cluster = get_current_context()
+        get_auth_token(env, current_cluster, zalando_kubectl.access_request._has_any_state_file(current_cluster))
         return
 
     token = auth_token()
@@ -186,3 +194,7 @@ def get_current_namespace():
 def get_current_context():
     config = read_config()
     return config.get("current-context")
+
+
+def get_context(env: Environment):
+    return env.kube_context or get_current_context()
