@@ -15,12 +15,14 @@ from pathlib import Path
 import click
 import clickclick
 import humanize
+import jinja2
 import requests
 import yaml
 import zalando_kubectl
 import zalando_kubectl.kube_permissions
 import zalando_kubectl.secrets
 import zalando_kubectl.traffic
+import zalando_kubectl.priv_api
 import zalando_kubectl.utils as utils
 from clickclick import Action, info, print_table, OutputFormat
 from dateutil.parser import parse
@@ -38,6 +40,7 @@ from zalando_kubectl.utils import (
 
 from . import access_request
 from . import completion as comp
+from . import generate as gen
 from . import kube_config, registry
 from .templating import copy_template, prepare_variables, read_senza_variables
 
@@ -102,6 +105,7 @@ def click_cli(ctx, env: Environment):
     if ctx.invoked_subcommand != completion.name:
         env.kubectl.download()
         env.kubelogin.download()
+        env.zalando_aws_cli.download()
     pass
 
 
@@ -112,7 +116,9 @@ def get_registry_url(config):
     try:
         return config["cluster_registry"].rstrip("/")
     except KeyError:
-        raise Exception("Cluster registry URL missing, please reconfigure zkubectl")
+        raise Exception(
+            "Cluster registry URL missing, please reconfigure zkubectl by running `zalando-cli-bundle configure`"
+        )
 
 
 def fix_url(url):
@@ -182,16 +188,22 @@ def login(config, cluster_or_url: str):
 @click_cli.command("configure")
 @click.option("--cluster-registry", required=False, help="Cluster registry URL")
 @click.option("--okta-auth", required=False, help="Okta Auth URL")
+@click.option("--privileged-access-api", required=False, help="Privileged Access API URL")
 @click.pass_obj
-def configure(env: Environment, cluster_registry, okta_auth):
-    """Set the Cluster Registry and/or Okta OIDC Issuer URL."""
-    if not any([cluster_registry, okta_auth]):
-        print("Missing arguments. Provide Cluster Registry and/or Okta OIDC Issuer URL.")
+def configure(env: Environment, cluster_registry, okta_auth, privileged_access_api):
+    """Set the Cluster Registry, Okta OIDC Issuer URL or Privileged Access API URL."""
+    if not any([cluster_registry, okta_auth, privileged_access_api]):
+        print(
+            "Missing arguments. Provide at least one of Cluster Registry, Okta OIDC Issuer URL or "
+            + "Privileged Access API URL. Alternatively you can run `zalando-cli-bundle configure`."
+        )
 
     if cluster_registry:
         env.config["cluster_registry"] = cluster_registry
     if okta_auth:
         env.config["okta_auth"] = okta_auth
+    if privileged_access_api:
+        env.config["privileged_access_api"] = privileged_access_api
 
     env.store_config()
 
@@ -224,7 +236,7 @@ def dashboard(env: Environment):
     thread = threading.Thread(target=_open_dashboard_in_browser)
     # start short-lived background thread to allow running "kubectl proxy" in main thread
     thread.start()
-    kube_config.update_token()
+    kube_config.update_token(env)
     env.kubectl.exec("proxy")
 
 
@@ -265,7 +277,7 @@ def opsview(env: Environment):
     else:
         opts = ["--net=host"]
     subprocess.Popen(["docker", "run", "--rm", "-i"] + opts + [image_name])
-    kube_config.update_token()
+    kube_config.update_token(env)
     env.kubectl.exec("proxy", "--accept-hosts=.*")
 
 
@@ -274,7 +286,7 @@ def opsview(env: Environment):
 @click.pass_obj
 def logtail(env: Environment, args):
     """Tail multiple pods and containers"""
-    kube_config.update_token()
+    kube_config.update_token(env)
     env.stern.exec(*args)
 
 
@@ -328,16 +340,21 @@ def credentials(env: Environment, cluster):
 
 @click_cli.command("login")
 @click.argument("cluster", required=False)
-@click.option("--okta/--no-okta", default=False, help="Login with Okta")
+@click.option("--okta/--no-okta", default=False, help="Login with Okta", envvar="ZKUBECTL_USE_OKTA")
+@click.option(
+    "--force-refresh/--no-force-refresh",
+    default=False,
+    help="When using Okta, refresh the token regardless of its expiration time",
+)
 @click.pass_obj
-def do_login(env: Environment, cluster, okta):
+def do_login(env: Environment, cluster, okta, force_refresh):
     """Login to a specific cluster"""
     url, alias = login(env.config, cluster)
     env.store_config()
 
     token = auth_token()
     if okta:
-        token = kube_config.get_auth_token(env, cluster)
+        token = kube_config.get_auth_token(env, cluster, force_refresh)
 
     zalando_kubectl.kube_permissions.check_cluster_permissions(url, alias, token)
 
@@ -348,9 +365,11 @@ def do_login(env: Environment, cluster, okta):
 @click_cli.command("encrypt")
 @click.option("--cluster", help="Cluster ID or alias")
 @click.option("--strip/--no-strip", default=True, help="Remove the trailing newline from the data, enabled by default")
-@click.option("--kms-keyid", help="KMS key ID")
+@click.option("--kms-keyid", help="The KMS key ID that should be used to encrypt the payload, optional")
+@click.option("--okta/--no-okta", default=False, help="Login with Okta", envvar="ZKUBECTL_USE_OKTA")
+@click.option("--role", help="The role to use in the target account, optional (only used when using --okta)")
 @click.pass_obj
-def encrypt(env: Environment, cluster, kms_keyid, strip):
+def encrypt(env: Environment, cluster, kms_keyid, strip, role, okta):
     """Encrypt a value for use in a deployment configuration"""
     registry_url = get_registry_url(env.config)
     if cluster:
@@ -358,10 +377,6 @@ def encrypt(env: Environment, cluster, kms_keyid, strip):
     else:
         cluster_metadata = registry.get_cluster_with_params(registry_url, api_server_url=utils.get_api_server_url(env))
 
-    if not kms_keyid:
-        local_id = cluster_metadata["id"].split(":")[-1]
-        kms_keyid = "alias/{}-deployment-secret".format(local_id)
-
     if click.get_text_stream("stdin").isatty():
         plain_text = click.prompt("Data to encrypt", hide_input=True).encode()
     else:
@@ -370,15 +385,41 @@ def encrypt(env: Environment, cluster, kms_keyid, strip):
     if strip:
         plain_text = plain_text.rstrip(b"\r\n")
 
-    encrypted = zalando_kubectl.secrets.encrypt(cluster_metadata, kms_keyid, plain_text)
+    if okta:
+        encrypted = (
+            zalando_kubectl.secrets.encrypt_with_okta(env, cluster_metadata, kms_keyid, role, strip, plain_text)
+            .decode("utf-8")
+            .strip()
+        )
+    else:
+        if not kms_keyid:
+            local_id = cluster_metadata["id"].split(":")[-1]
+            kms_keyid = "alias/{}-deployment-secret".format(local_id)
+
+        encrypted = zalando_kubectl.secrets.encrypt(cluster_metadata, kms_keyid, plain_text)
+
     print(encrypted)
 
 
 @click_cli.command("decrypt")
 @click.argument("encrypted_value", required=True)
+@click.option("--okta/--no-okta", default=False, help="Login with Okta", envvar="ZKUBECTL_USE_OKTA")
+@click.option("--role", help="The role to use in the target account, optional (only used when using --okta)")
 @click.pass_obj
-def decrypt(env: Environment, encrypted_value):
+def decrypt(env: Environment, role, okta, encrypted_value):
     """Decrypt a value encrypted with zkubectl encrypt"""
+    if okta:
+        decrypted = zalando_kubectl.secrets.decrypt_with_okta(env, role, encrypted_value)
+
+        # zalando-aws-cli v0.4.0 always adds a newline to the decrypted output:
+        #
+        #   https://github.bus.zalan.do/Base-Infrastructure/zalando-aws-cli/blob/v0.4.0/cmd/secrets.go#L218
+        #
+        # To ensure values are returned in the same way as with non-Okta encrypted values, we strip the last character.
+        decrypted = decrypted[0 : len(decrypted) - 1]
+
+        return sys.stdout.buffer.write(decrypted)
+
     registry_url = get_registry_url(env.config)
 
     parts = encrypted_value.split(":")
@@ -435,7 +476,7 @@ def _validate_weight(_ctx, _param, value):
 @click.argument("weight", required=False, type=float, callback=_validate_weight)
 @click.pass_obj
 def traffic(env: Environment, force, no_wait, timeout, stackset_name, stack, weight):
-    kube_config.update_token()
+    kube_config.update_token(env)
 
     try:
         if stack is None:
@@ -466,6 +507,194 @@ def traffic(env: Environment, force, no_wait, timeout, stackset_name, stack, wei
         raise click_exc
 
 
+@click_cli.command(
+    "generate",
+    help="""Generate boilerplate of kubernetes manifests. Implemented types:
+
+    - deployment # generate deployment resource with CLI input
+
+    - fabric     # generate fabricgateway resource with CLI input and from API repository
+
+    - ingress    # generate ingress resource with CLI input
+
+    - stackset   # generate stackset resource with CLI input
+""",
+)
+@click.argument("typ", required=True)
+@click.option(
+    "--application",
+    "-a",
+    help="Application ID",
+    required=True,
+)
+@click.option(
+    "--component",
+    "-c",
+    help="Component to add as label",
+    required=False,
+)
+@click.option(
+    "--host",
+    help="Ingress hosts as comma separated list, that will be exposed by DNS",
+    required=False,
+)
+@click.option(
+    "--backendport",
+    help="Ingress backendport to generate your Ingress with service",
+    required=False,
+)
+@click.option(
+    "--scopes",
+    help="Ingress tokeninfo scopes as comma separated list to allow to access (defaults to uid, which means all valid zalando employees and services)",
+    required=False,
+)
+@click.option(
+    "--ui",
+    help="Ingresss to a UI Application for employees",
+    required=False,
+    is_flag=True,
+)
+@click.option(
+    "--team",
+    "-t",
+    help="Radical Agility Team name that is used to find team members to generate fabricgateway admin access",
+    required=False,
+)
+@click.option(
+    "--file",
+    "-f",
+    help="File to OpenAPI Spec to generate fabricgateway",
+    required=False,
+)
+@click.option(
+    "--image",
+    help="docker image",
+    required=False,
+)
+@click.option(
+    "--cpu",
+    help="CPU resource requests",
+    required=False,
+)
+@click.option(
+    "--memory",
+    help="Memory resource requests",
+    required=False,
+)
+@click.option(
+    "--cluster",
+    help="Cluster Alias name for example: 'fashion-store'",
+    required=False,
+)
+@click.option(
+    "--config",
+    help="generate configmap",
+    required=False,
+    is_flag=True,
+)
+@click.option(
+    "--secret",
+    help="generate static secret config",
+    required=False,
+    is_flag=True,
+)
+@click.option(
+    "--autoscaling",
+    help="generate horizontal pod autoscaling",
+    required=False,
+    is_flag=True,
+)
+@click.option(
+    "--debug",
+    "-d",
+    help="Enable debug",
+    required=False,
+    is_flag=True,
+)
+@click.pass_obj
+def generate(
+    env: Environment,
+    typ,
+    application,
+    component,
+    host,
+    backendport,
+    scopes,
+    ui,
+    team,
+    file,
+    image,
+    cpu,
+    memory,
+    cluster,
+    config,
+    secret,
+    autoscaling,
+    debug,
+):
+    try:
+        if typ is None:
+            raise click.UsageError("No typ information found")
+
+        if application is None:
+            raise click.UsageError("You must specify the application")
+
+        if typ == "fabric":
+            token = auth_token()
+            gen.fabric(token, application, component, team, backendport, file, debug)
+
+        elif typ == "ingress":
+            if not host:
+                raise click.UsageError("--host required for ingress")
+            gen.ingress(application, component, team, host, backendport, scopes, ui, debug)
+
+        elif typ == "deployment":
+            gen.deployment(
+                application,
+                component,
+                team,
+                backendport,
+                scopes,
+                image,
+                cpu,
+                memory,
+                cluster,
+                config,
+                secret,
+                autoscaling,
+                debug,
+            )
+
+        elif typ == "stackset":
+            if not host:
+                raise click.UsageError("--host required for stackset")
+            gen.stackset(
+                application,
+                component,
+                team,
+                host,
+                backendport,
+                scopes,
+                image,
+                cpu,
+                memory,
+                cluster,
+                config,
+                secret,
+                ui,
+                autoscaling,
+                debug,
+            )
+
+        else:
+            raise click.UsageError("Wrong typ information found: {}".format(typ))
+
+    except subprocess.CalledProcessError as e:
+        click_exc = click.ClickException(e.stderr.decode("utf-8"))
+        click_exc.exit_code = e.returncode
+        raise click_exc
+
+
 @click_cli.group(
     name="cluster-update",
     cls=DecoratingGroup,
@@ -598,7 +827,7 @@ def tunnel(env: Environment, address, target, ports):
     signal.signal(signal.SIGTERM, remove_pods)
 
     # refresh token
-    kube_config.update_token()
+    kube_config.update_token(env)
 
     # generate socat container specs for each port
     containers = generate_containers(target, ports, base_port, timeout)
@@ -670,7 +899,7 @@ def latest_socat_image():
 
     image = response.json()
 
-    return "{}/{}/{}:{}".format(docker_registry, team, artifact, image["tags"][-1])
+    return "{}/{}/{}:{}".format(docker_registry, team, artifact, sorted(image["tags"])[-1])
 
 
 def generate_containers(target, ports, base_port, timeout):
@@ -839,14 +1068,31 @@ def cluster_access_check_environment(env: Environment):
 
 @cluster_access.command("request")
 @click.option("--emergency", is_flag=True, help="Request emergency access to the cluster")
-@click.option("-i", "--incident", help="Opsgenie incident number or URL, required with --emergency", type=str)
-@click.option("-u", "--user", help="User that should be given access, defaults to the current user", required=False)
+@click.option("--okta/--no-okta", default=False, help="Login with Okta", envvar="ZKUBECTL_USE_OKTA")
+@click.option("-i", "--incident", help="Opsgenie incident number or URL, [required] with --emergency", type=str)
+@click.option(
+    "-u",
+    "--user",
+    help="User that should be given access, defaults to the current user, [invalid] with --okta",
+    required=False,
+)
+@click.option(
+    "-t",
+    "--timeout",
+    help="Duration, in seconds, to wait for the request to be approved, only valid with --okta",
+    default=600,
+    required=False,
+    type=int,
+)
 @click.argument("reason", nargs=-1, required=True)
 @click.pass_obj
-def request_cluster_access(env: Environment, emergency, incident, user, reason):
+def request_cluster_access(env: Environment, emergency, incident, user, reason, okta, timeout):
     """Request access to the cluster"""
     cluster_access_check_environment(env)
 
+    if user and okta:
+        raise click.UsageError("When using Okta, you cannot specify a username.")
+
     if emergency:
         if not incident:
             raise click.UsageError("You must specify an incident ticket [--incident] when requesting emergency access")
@@ -863,12 +1109,16 @@ def request_cluster_access(env: Environment, emergency, incident, user, reason):
             reference_url = "https://zalando.app.opsgenie.com/alert/show/" + incident
         else:
             raise click.UsageError("You must provide a valid incident number or ID, alert ID or an Opsgenie URL")
-        make_emergency_access_request(env, reference_url, user, reason)
+        make_emergency_access_request(env, reference_url, user, reason, okta, timeout)
     else:
-        make_manual_access_request(env, user, reason)
+        make_manual_access_request(env, user, reason, okta, timeout)
 
 
-def make_emergency_access_request(env: Environment, reference_url, user, reason):
+def make_emergency_access_request(env: Environment, reference_url, user, reason, okta, timeout):
+    if okta:
+        access_request.create_with_okta(env, "emergency", reference_url, " ".join(reason), timeout)
+        return
+
     user = access_request_username(user)
     access_request.create(env, "emergency", reference_url, user, " ".join(reason))
     click.echo(
@@ -877,7 +1127,11 @@ def make_emergency_access_request(env: Environment, reference_url, user, reason)
     )
 
 
-def make_manual_access_request(env: Environment, user_option, reason):
+def make_manual_access_request(env: Environment, user_option, reason, okta, timeout):
+    if okta:
+        access_request.create_with_okta(env, "privileged", None, " ".join(reason), timeout)
+        return
+
     user = access_request_username(user_option)
     access_request.create(env, "manual", None, user, " ".join(reason))
     click.echo("Requested manual access for {}.".format(user))
@@ -890,12 +1144,23 @@ def make_manual_access_request(env: Environment, user_option, reason):
 
 
 @cluster_access.command("approve")
-@click.argument("username", required=True)
+@click.argument("username_or_request_key", required=True)
+@click.option("--okta/--no-okta", default=False, help="Login with Okta", envvar="ZKUBECTL_USE_OKTA")
+@click.option("--yes/--no-yes", default=False, help="Approve the given access request without prompting again.")
 @click.pass_obj
-def approve_access_request(env: Environment, username):
-    """Approve a manual access request"""
+def approve_access_request(env: Environment, username_or_request_key, okta, yes):
+    """Approve a manual access request
+
+    USERNAME_OR_REQUEST_KEY is the username or request_key (when using --okta) to approve access for.
+    """
     cluster_access_check_environment(env)
 
+    if okta:
+        access_request.approve_with_okta(env, username_or_request_key, yes)
+        return
+
+    username = username_or_request_key
+
     if not username:
         username = click.prompt("User that should receive access")
 
@@ -912,14 +1177,23 @@ def approve_access_request(env: Environment, username):
 
 
 @cluster_access.command("list")
+@click.option("--okta/--no-okta", default=False, help="Login with Okta", envvar="ZKUBECTL_USE_OKTA")
+@click.option("--request-key", help="The request key that identifies your privileged access request")
 @click.option(
     "-o", "--output", type=click.Choice(["text", "json"]), default="text", help="Command output format (default: text)"
 )
 @click.pass_obj
-def list_cluster_requests(env: Environment, output):
+def list_cluster_requests(env: Environment, okta, request_key, output):
     """List all current pending/approved access requests for the cluster"""
     cluster_access_check_environment(env)
 
+    if request_key and not okta:
+        raise click.UsageError("Request Key is only valid when using Okta.")
+
+    if okta:
+        access_request.list_with_okta(env, request_key)
+        return
+
     all_requests = access_request.get_all(env)
     access_requests = sorted(all_requests.values(), key=lambda r: r["user"])
 
@@ -961,7 +1235,7 @@ def restart_pods(env: Environment, target):
 
 def find_cmd(cmdline):
     while len(cmdline) > 0:
-        if cmdline[0] in ("-n", "--namespace", "--context"):
+        if any(cmdline[0].startswith(flag) for flag in ("-n", "--namespace", "--context")):
             if "=" in cmdline[0]:
                 cmdline = cmdline[1:]
             else:
@@ -1037,7 +1311,7 @@ def main():
         elif not cmd or cmd in click_cli.get_help_option_names(ctx):
             print_help(ctx)
         else:
-            kube_config.update_token()
+            kube_config.update_token(ctx.obj)
             ctx.obj.kubectl.exec(*sys.argv[1:])
     except KeyboardInterrupt:
         debug_print_exc()
