@@ -1,11 +1,18 @@
 import base64
+import subprocess
 import zalando_aws_cli.api
-from zalando_kubectl.utils import auth_token
+from zalando_kubectl.utils import auth_token, Environment, ExternalBinary
 
 _ENCRYPT_ROLES = ("ReadOnly", "Deployer", "Manual", "Emergency", "Administrator", "PowerUser")
 _DECRYPT_ROLES = ("Manual", "Emergency", "Administrator", "PowerUser")
 
 
+def zalando_aws_cli_run(zalando_aws_cli: ExternalBinary, *cmd):
+    return zalando_aws_cli.run(
+        cmd, check=True, stdout=subprocess.PIPE, forward_context=False, forward_namespace=False
+    ).stdout
+
+
 def find_aws_role(token, account_id, roles):
     """Returns the best matching AWS role for the provided account"""
     user_roles = zalando_aws_cli.api.get_roles(token)
@@ -64,7 +71,34 @@ def encrypt(cluster_metadata, kms_keyid, plain_text_blob):
     return secret.format(account_name=account_name, encrypted=encrypted.decode())
 
 
+def encrypt_with_okta(env: Environment, cluster_metadata, kms_keyid, role, strip, plain_text):
+    cmdline = [
+        "--target-account={}".format(cluster_metadata["alias"]),
+        "encrypt",
+        plain_text,
+    ]
+    if not strip:
+        cmdline.append("--strip=false")
+    if kms_keyid:
+        cmdline.append("--kms-keyid={}".format(kms_keyid))
+    if role:
+        cmdline.append("--role={}".format(role))
+
+    return zalando_aws_cli_run(env.zalando_aws_cli, *cmdline)
+
+
 def decrypt(cluster_metadata, encrypted_value):
     kms = kms_for_cluster(cluster_metadata, allow_decrypt=True)
     result = kms.decrypt(CiphertextBlob=encrypted_value)
     return result["Plaintext"]
+
+
+def decrypt_with_okta(env: Environment, role, encrypted_value):
+    cmdline = [
+        "decrypt",
+        encrypted_value,
+    ]
+    if role:
+        cmdline.append("--role={}".format(role))
+
+    return zalando_aws_cli_run(env.zalando_aws_cli, *cmdline)
