@@ -6,8 +6,8 @@ import string
 import subprocess
 import sys
 import tarfile
-from zipfile import ZipFile
 from pathlib import Path
+from zipfile import ZipFile
 
 import click
 import jwt
@@ -16,6 +16,7 @@ import stups_cli
 import stups_cli.config
 import zign.api
 from clickclick import Action, AliasedGroup
+from subprocess import check_output
 
 from . import (
     APP_NAME,
@@ -24,6 +25,8 @@ from . import (
     KUBECTL_SHA512,
     KUBELOGIN_VERSION,
     KUBELOGIN_SHA256,
+    ZALANDO_AWS_CLI_VERSION,
+    ZALANDO_AWS_CLI_SHA256,
     STERN_VERSION,
     STERN_SHA256,
 )
@@ -110,27 +113,32 @@ class ExternalBinary:
                 if m.hexdigest() != self.checksum[platform]:
                     act.fatal_error("CHECKSUM MISMATCH")
 
-                # add random suffix so we can do an atomic write and rename
-                random_suffix = "".join(random.choice(string.ascii_letters + string.digits) for _ in range(8))
-                local_file = binary.with_name("{}.download-{}".format(binary.name, random_suffix))
-                with local_file.open("wb") as fd:
-                    fd.write(self.extract(data))
-                    fd.flush()
-                local_file.chmod(0o755)
-                local_file.rename(binary)
+                self.create_binary(binary, self.extract(data))
 
         return str(binary)
 
-    def run(self, args, input=None, stdout=None, stderr=None, check=False):
+    def create_binary(self, binary, data):
+        # add random suffix so we can do an atomic write and rename
+        random_suffix = "".join(random.choice(string.ascii_letters + string.digits) for _ in range(8))
+        local_file = binary.with_name("{}.download-{}".format(binary.name, random_suffix))
+        with local_file.open("wb") as fd:
+            fd.write(data)
+            fd.flush()
+        local_file.chmod(0o755)
+        local_file.rename(binary)
+
+    def run(
+        self, args, input=None, stdout=None, stderr=None, check=False, forward_context=True, forward_namespace=True
+    ):
         if self.exec_name is not None:
             exec_name = self.exec_name
         else:
             exec_name = self.name
 
         cmdline = [exec_name]
-        if self.env.namespace:
+        if forward_namespace and self.env.namespace:
             cmdline += ["-n", self.env.namespace]
-        if self.env.kube_context:
+        if forward_context and self.env.kube_context:
             cmdline += ["--context", self.env.kube_context]
         cmdline.extend(args)
 
@@ -175,9 +183,7 @@ class Environment:
             STERN_VERSION,
             hashlib.sha256,
             STERN_SHA256,
-            extract_tgz(
-                "stern_{version}_{os}_{arch}/stern".format(version=STERN_VERSION, os=sys.platform, arch="amd64")
-            ),
+            extract_tgz("stern"),
         )
 
         kubelogin_template = "https://github.com/int128/kubelogin/releases/download/{version}/kubelogin_{os}_{arch}.zip"
@@ -195,6 +201,19 @@ class Environment:
             ),
         )
 
+        zalando_aws_cli_template = (
+            "https://cloud-utils.s3.eu-central-1.amazonaws.com/zalando-aws-cli-{version}-{os}-{arch}.tar.gz"
+        )
+        self.zalando_aws_cli = ExternalBinary(
+            self,
+            "zalando-aws-cli",
+            zalando_aws_cli_template,
+            ZALANDO_AWS_CLI_VERSION,
+            hashlib.sha256,
+            ZALANDO_AWS_CLI_SHA256,
+            extract_tgz("./zalando-aws-cli"),
+        )
+
     def store_config(self):
         stups_cli.config.store_config(self.config, APP_NAME)
 
